/******************************************************************************
 *                                                                            *
 * File:   setfd32.c          Version  2.00            Date: 1999-04-03       *
 *                                                                            *
 * Copyright (C) 1993-1999 by Kostis Netzwerkberatung                         *
 * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056, FAX 881058            *
 * kosta@kostis.net (Kosta Kostis), http://www.kostis.net/                    *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * setfd32 sets file date and time                                            *
 *                                                                            *
 * Syntax:                                                                    *
 *	setfd32 file                       sets current date and time         *
 *	setfd32 file hh:mm:ss             sets time only                      *
 *	setfd32 file hh:mm:ss dd-mm-ccyy  sets date and time                  *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     1999-04-03: KK V2.00                                                   *
 *        - ported to win32                                                   *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include <winbase.h>

#include "setfd32.h"

/******************************************************************************

	Function:
		ChangeDate

	Parameters:
		LPSYSTEMTIME	lpSystemTime	file time/date in UTC
		char		*datestr	date string (dd.mm.ccyy)

	Returns:
		nothing

	Remarks:
		No tests are performed. Invalid data will be copied.

 *****************************************************************************/

static	void	ChangeDate
(
	LPSYSTEMTIME	lpSystemTime,
	char		*datestr
)
{
	WORD		year ;

	/**********************************************************************

		attempt to take in account the 21st century

	 *********************************************************************/

	year = atoi (&datestr [6]) ;

	/**********************************************************************

		check if 4 or 2 digits are used

	 *********************************************************************/

	if (year < 1980)
	{
		/**************************************************************

			OK, we got only 2 digits, guess correct century

		 *************************************************************/

		if (year < 80)			/* no MS-DOS before 1980 */
			year += 2000 ;
		else
			year += 1900 ;
	}

	/**********************************************************************

		replace date members with date given

	 *********************************************************************/

	lpSystemTime -> wDay   = atoi (datestr) ;
	lpSystemTime -> wMonth = atoi (&datestr [3]) ;
	lpSystemTime -> wYear  = year ;
}

/******************************************************************************

	Function:
		ChangeTime

	Parameters:
		LPSYSTEMTIME	lpSystemTime	file time/date in UTC
		char		*timestr	time string (hh:mm:ss)

	Returns:
		nothing

	Remarks:
		No tests are performed. Invalid data will be copied.

 *****************************************************************************/

static	void	ChangeTime
(
	LPSYSTEMTIME	lpSystemTime,
	char		*timestr
)
{
	/**********************************************************************

		replace time members with time given

	 *********************************************************************/

	lpSystemTime -> wHour   = atoi (timestr) ;
	lpSystemTime -> wMinute = atoi (&timestr [3]) ;
	lpSystemTime -> wSecond = atoi (&timestr [6]) ;
}

/******************************************************************************

    main () does it all				;-)

 *****************************************************************************/

int	main
(
	int			argc,
	char			*argv		[]
)
{
	HANDLE			MODfile ;
	FILETIME		FileTime ;		/* file time (UTC) */
	FILETIME		LocalFileTime ;		/* local file time */
	SYSTEMTIME		LocalSysTime ;		/* local sys time  */

	/**********************************************************************

		check syntax first

	 *********************************************************************/

	if ((argc < 2) || (argc > 4))
	{
		fprintf (stderr, MSG_COPYRIGHT) ;
		fprintf (stderr, MSG_VERSION, PRGNAM, VERSION) ;
		fprintf (stderr, ERR_MSG_SYNTAX, PRGNAM) ;

		return (ERR_SYNTAX) ;
	}

	/**********************************************************************

		try to open file

	 *********************************************************************/

	MODfile = CreateFile
	(
		(LPCTSTR) argv [1],
		GENERIC_READ,
		FILE_SHARE_READ,
		(LPSECURITY_ATTRIBUTES) NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		(HANDLE) NULL
	) ;

	/**********************************************************************

		does this file exist?

	 *********************************************************************/

	if (MODfile == (HANDLE) NULL)
	{
		/**************************************************************

			try to create new file

		 *************************************************************/

		MODfile = CreateFile
		(
			(LPCTSTR) argv [1],
			GENERIC_WRITE,
			FILE_SHARE_WRITE,
			(LPSECURITY_ATTRIBUTES) NULL,
			CREATE_NEW,
			FILE_ATTRIBUTE_NORMAL,
			(HANDLE) NULL
		) ;

		/**************************************************************

			check if file creation was successful

		 *************************************************************/

		if (MODfile == (HANDLE) NULL)
		{
			fprintf (stderr, ERR_MSG_OPEN, PRGNAM, argv [1]) ;
			return (ERR_OPEN) ;
		}

		CloseHandle (MODfile) ;
		return (ERR_NONE) ;
	}

	/**********************************************************************

		file does exist

	 *********************************************************************/

	CloseHandle (MODfile) ;

	/**********************************************************************

		open file with write access (append)

	 *********************************************************************/

	MODfile = CreateFile
	(
		(LPCTSTR) argv [1],
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_WRITE,
		(LPSECURITY_ATTRIBUTES) NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		(HANDLE) NULL
	) ;

	/**********************************************************************

		check if this file can be written

	 *********************************************************************/

	if (MODfile == (HANDLE) NULL)
	{
		fprintf (stderr, ERR_MSG_OPEN, PRGNAM, argv [1]) ;
		return (ERR_OPEN) ;
	}

	/**********************************************************************

		retrieve original file date/time

	 *********************************************************************/

	GetFileTime (MODfile,
		(LPFILETIME) NULL, (LPFILETIME) NULL, &FileTime) ;

	/**********************************************************************

		convert UTC date/time in local system date/time

	 *********************************************************************/

	FileTimeToLocalFileTime (&FileTime, &LocalFileTime) ;
	FileTimeToSystemTime (&LocalFileTime, &LocalSysTime) ;

	/**********************************************************************

		modify what needs to be modified

	 *********************************************************************/

	switch (argc)
	{
		case 4:

			ChangeDate (&LocalSysTime, argv [3]) ;

		case 3:

			ChangeTime (&LocalSysTime, argv [2]) ;

			SystemTimeToFileTime (&LocalSysTime, &LocalFileTime) ;
			LocalFileTimeToFileTime (&LocalFileTime, &FileTime) ;
			break ;

		default:

			GetSystemTimeAsFileTime (&FileTime) ;
	}

	/**********************************************************************

		set new file date/time

	 *********************************************************************/

	SetFileTime (MODfile,
		(LPFILETIME) NULL, (LPFILETIME) NULL, &FileTime) ;

	/**********************************************************************

		close file, we're done

	 *********************************************************************/

	CloseHandle (MODfile) ;

	return (ERR_NONE) ;
}
